@@ -49,7 +49,7 @@ from bleak.exc import BleakDBusError, BleakDeviceNotFoundError, BleakError
 logger = logging.getLogger(__name__)
 
 # prevent tasks from being garbage collected
-_background_tasks = set[asyncio.Task[None]]()
+_background_tasks: set[asyncio.Task[None]] = set()
 
 
 class BleakClientBlueZDBus(BaseBleakClient):
@@ -103,6 +103,8 @@ class BleakClientBlueZDBus(BaseBleakClient):
         self._disconnect_monitor_event: Optional[asyncio.Event] = None
         # map of characteristic D-Bus object path to notification callback
         self._notification_callbacks: dict[str, NotifyCallback] = {}
+        # map of characteristic D-Bus path to AcquireNotify file descriptor
+        self._notification_fds: dict[str, int] = {}
 
         # used to override mtu_size property
         self._mtu_size: Optional[int] = None
@@ -270,7 +272,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                                     ],
                                 )
                             )
-                            assert reply
                             assert_reply(reply)
 
                             # REVIST: This leaves "Trusted" property set if we
@@ -304,7 +305,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                                     member="Connect",
                                 )
                             )
-                        assert reply
 
                         if reply.message_type == MessageType.ERROR:
                             # This error is often caused by RF interference
@@ -457,7 +457,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                         member="Disconnect",
                     )
                 )
-                assert reply
                 assert_reply(reply)
                 async with async_timeout(10):
                     await self._disconnecting_event.wait()
@@ -487,7 +486,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                 body=[defs.DEVICE_INTERFACE, "Paired"],
             )
         )
-        assert reply
         assert_reply(reply)
         if reply.body[0].value:
             logger.debug("BLE device @ %s is already paired", self.address)
@@ -504,7 +502,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                 body=[defs.DEVICE_INTERFACE, "Trusted", Variant("b", True)],
             )
         )
-        assert reply
         assert_reply(reply)
 
         logger.debug("Pairing to BLE device @ %s", self.address)
@@ -517,7 +514,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                 member="Pair",
             )
         )
-        assert reply
         assert_reply(reply)
 
         # For resolvable private addresses, the address will
@@ -566,7 +562,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                     body=[device_path],
                 )
             )
-            assert reply
             assert_reply(reply)
         except BleakDBusError as e:
             if e.dbus_error == "org.bluez.Error.DoesNotExist":
@@ -630,7 +625,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                 body=[{}],
             )
         )
-        assert reply
         assert_reply(reply)
 
         # we aren't actually using the write or notify, we just want the MTU
@@ -748,8 +742,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                 )
             )
 
-            assert reply
-
             if reply.error_name == "org.bluez.Error.InProgress":
                 logger.debug("retrying characteristic ReadValue due to InProgress")
                 # Avoid calling in a tight loop. There is no dbus signal to
@@ -798,8 +790,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                 )
             )
 
-            assert reply
-
             if reply.error_name == "org.bluez.Error.InProgress":
                 logger.debug("retrying descriptor ReadValue due to InProgress")
                 # Avoid calling in a tight loop. There is no dbus signal to
@@ -841,8 +831,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                 )
             )
 
-            assert reply
-
             if reply.error_name == "org.bluez.Error.InProgress":
                 logger.debug("retrying characteristic WriteValue due to InProgress")
                 # Avoid calling in a tight loop. There is no dbus signal to
@@ -888,8 +876,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
                 )
             )
 
-            assert reply
-
             if reply.error_name == "org.bluez.Error.InProgress":
                 logger.debug("retrying descriptor WriteValue due to InProgress")
                 # Avoid calling in a tight loop. There is no dbus signal to
@@ -904,6 +890,39 @@ class BleakClientBlueZDBus(BaseBleakClient):
             "Write Descriptor %s | %s: %s", descriptor.handle, descriptor.obj[0], data
         )
 
+    def _register_notify_fd_reader(
+        self, char_path: str, fd: int, callback: NotifyCallback
+    ) -> None:
+        loop = asyncio.get_running_loop()
+
+        def on_data():
+            try:
+                data = os.read(fd, 1024)
+                if not data:
+                    raise RuntimeError("Unexpected EOF on notification file handle")
+            except Exception as e:
+                logger.debug(
+                    "AcquireNotify: Read error on fd %d: %s. Notifications have been stopped.",
+                    fd,
+                    e,
+                )
+                try:
+                    loop.remove_reader(fd)
+                except RuntimeError:
+                    # Run loop is closed
+                    pass
+                try:
+                    os.close(fd)
+                except OSError:
+                    # Bad file descriptor
+                    pass
+                self._notification_fds.pop(char_path, None)
+                return
+
+            callback(bytearray(data))
+
+        loop.add_reader(fd, on_data)
+
     @override
     async def start_notify(
         self,
@@ -914,20 +933,48 @@ class BleakClientBlueZDBus(BaseBleakClient):
         """
         Activate notifications/indications on a characteristic.
         """
-        self._notification_callbacks[characteristic.obj[0]] = callback
-
         assert self._bus is not None
 
-        reply = await self._bus.call(
-            Message(
-                destination=defs.BLUEZ_SERVICE,
-                path=characteristic.obj[0],
-                interface=defs.GATT_CHARACTERISTIC_INTERFACE,
-                member="StartNotify",
-            )
+        # If using StartNotify and calling a read on the same
+        # characteristic, BlueZ will return the response as
+        # both a notification and read, duplicating the message.
+        # Using AcquireNotify on supported characteristics avoids this.
+        # However, using the preferred AcquireNotify requires that devices
+        # correctly indicate "notify" and/or "indicate" properties. If they
+        # don't, we fall back to StartNotify.
+        use_notify_acquire = "NotifyAcquired" in characteristic.obj[1]
+        logger.debug(
+            'using "%s" for notifications on characteristic %d',
+            "AcquireNotify" if use_notify_acquire else "StartNotify",
+            characteristic.handle,
         )
-        assert reply
-        assert_reply(reply)
+        if use_notify_acquire:
+            reply = await self._bus.call(
+                Message(
+                    destination=defs.BLUEZ_SERVICE,
+                    path=characteristic.obj[0],
+                    interface=defs.GATT_CHARACTERISTIC_INTERFACE,
+                    member="AcquireNotify",
+                    body=[{}],
+                    signature="a{sv}",
+                )
+            )
+            assert_reply(reply)
+
+            unix_fd = reply.unix_fds[0]
+            self._notification_fds[characteristic.obj[0]] = unix_fd
+            self._register_notify_fd_reader(characteristic.obj[0], unix_fd, callback)
+        else:
+            self._notification_callbacks[characteristic.obj[0]] = callback
+            reply = await self._bus.call(
+                Message(
+                    destination=defs.BLUEZ_SERVICE,
+                    path=characteristic.obj[0],
+                    interface=defs.GATT_CHARACTERISTIC_INTERFACE,
+                    member="StartNotify",
+                )
+            )
+            assert_reply(reply)
 
     @override
     async def stop_notify(self, characteristic: BleakGATTCharacteristic) -> None:
@@ -942,15 +989,39 @@ class BleakClientBlueZDBus(BaseBleakClient):
 
         assert self._bus is not None
 
-        reply = await self._bus.call(
-            Message(
-                destination=defs.BLUEZ_SERVICE,
-                path=characteristic.obj[0],
-                interface=defs.GATT_CHARACTERISTIC_INTERFACE,
-                member="StopNotify",
+        if "NotifyAcquired" in characteristic.obj[1]:
+            logger.debug(
+                "Closing notification fd for characteristic %d", characteristic.handle
             )
-        )
-        assert reply
-        assert_reply(reply)
+            fd = self._notification_fds.pop(characteristic.obj[0], None)
 
-        self._notification_callbacks.pop(characteristic.obj[0], None)
+            if fd is None:
+                logger.debug(
+                    "No notification fd found for characteristic %d",
+                    characteristic.handle,
+                )
+            else:
+                loop = asyncio.get_running_loop()
+                try:
+                    loop.remove_reader(fd)
+                except RuntimeError:
+                    # Run loop is closed
+                    pass
+                try:
+                    os.close(fd)
+                except OSError as e:
+                    logger.debug("Failed to remove file descriptor %d: %s", fd, e)
+        else:
+            logger.debug(
+                "Calling StopNotify for characteristic %d", characteristic.handle
+            )
+            reply = await self._bus.call(
+                Message(
+                    destination=defs.BLUEZ_SERVICE,
+                    path=characteristic.obj[0],
+                    interface=defs.GATT_CHARACTERISTIC_INTERFACE,
+                    member="StopNotify",
+                )
+            )
+            assert_reply(reply)
+            self._notification_callbacks.pop(characteristic.obj[0], None)
