@@ -61,6 +61,8 @@ class BleakClientP4Android(BaseBleakClient):
         self.__gatt = None
         self.__mtu = 23
 
+        self.__callbacks = None
+
     # Connectivity methods
 
     @override
@@ -281,7 +283,7 @@ class BleakClientP4Android(BaseBleakClient):
                     descriptor = BleakGATTDescriptor(
                         java_descriptor,
                         characteristic.handle + 1 + descriptor_index,
-                        self.obj.getUuid().toString(),
+                        java_descriptor.getUuid().toString(),
                         characteristic,
                     )
                     services.add_descriptor(descriptor)
