@@ -15,6 +15,7 @@ if sys.version_info < (3, 12):
 else:
     from typing import override
 
+from winrt.windows.devices.bluetooth import BluetoothAdapter
 from winrt.windows.devices.bluetooth.advertisement import (
     BluetoothLEAdvertisementReceivedEventArgs,
     BluetoothLEAdvertisementType,
@@ -23,6 +24,7 @@ from winrt.windows.devices.bluetooth.advertisement import (
     BluetoothLEAdvertisementWatcherStoppedEventArgs,
     BluetoothLEScanningMode,
 )
+from winrt.windows.devices.radios import RadioState
 from winrt.windows.foundation import EventRegistrationToken
 
 from bleak.assigned_numbers import AdvertisementDataType
@@ -32,7 +34,11 @@ from bleak.backends.scanner import (
     BaseBleakScanner,
 )
 from bleak.backends.winrt.util import assert_mta
-from bleak.exc import BleakError
+from bleak.exc import (
+    BleakBluetoothNotAvailableError,
+    BleakBluetoothNotAvailableReason,
+    BleakError,
+)
 from bleak.uuids import normalize_uuid_str
 
 logger = logging.getLogger(__name__)
@@ -54,7 +60,7 @@ class RawAdvData(NamedTuple):
     Platform-specific advertisement data.
 
     Windows does not combine advertising data with type SCAN_RSP with other
-    advertising data like other platforms, so se have to do it ourselves.
+    advertising data like other platforms, so we have to do it ourselves.
     """
 
     adv: Optional[BluetoothLEAdvertisementReceivedEventArgs]
@@ -239,6 +245,26 @@ class BleakScannerWinRT(BaseBleakScanner):
         # there is nothing pumping a Windows message loop.
         await assert_mta()
 
+        adapter = await BluetoothAdapter.get_default_async()
+        if adapter is None:
+            raise BleakBluetoothNotAvailableError(
+                "No Bluetooth adapter found",
+                BleakBluetoothNotAvailableReason.NO_BLUETOOTH,
+            )
+
+        if not adapter.is_central_role_supported:
+            raise BleakBluetoothNotAvailableError(
+                "BLE 'central' role not supported on this adapter",
+                BleakBluetoothNotAvailableReason.NO_BLE_CENTRAL_ROLE,
+            )
+
+        radio = await adapter.get_radio_async()
+        if radio.state != RadioState.ON:
+            raise BleakBluetoothNotAvailableError(
+                "Bluetooth radio is not powered on. Turn on Bluetooth and try again.",
+                BleakBluetoothNotAvailableReason.POWERED_OFF,
+            )
+
         # start with fresh list of discovered devices
         self.seen_devices = {}
         self._advertisement_pairs.clear()
