@@ -4,15 +4,15 @@ Example showing how to use BleakClient.mtu_size
 
 import asyncio
 
-from bleak import BleakClient, BleakScanner
+from bleak import BleakBackend, BleakClient, BleakScanner
 from bleak.backends.scanner import AdvertisementData, BLEDevice
 
 # replace with real characteristic UUID
 CHAR_UUID = "00000000-0000-0000-0000-000000000000"
 
 
-async def main():
-    queue = asyncio.Queue[BLEDevice]()
+async def main() -> None:
+    queue: asyncio.Queue[BLEDevice] = asyncio.Queue()
 
     def callback(device: BLEDevice, adv: AdvertisementData) -> None:
         # can use advertising data to filter here
@@ -26,7 +26,7 @@ async def main():
         # BlueZ doesn't have a proper way to get the MTU, so we have this hack.
         # If this doesn't work for you, you can set the client._mtu_size attribute
         # to override the value instead.
-        if client._backend.__class__.__name__ == "BleakClientBlueZDBus":  # type: ignore
+        if client.backend_id == BleakBackend.BLUEZ_DBUS:
             await client._backend._acquire_mtu()  # type: ignore
 
         print("MTU:", client.mtu_size)
